@@ -359,6 +359,7 @@ void KPluginSelector::load()
         d->pluginModel->setData(index, pluginEntry->pluginInfo.isPluginEnabled(), Qt::CheckStateRole);
     }
 
+    static_cast<KPluginSelector::Private::PluginDelegate*>(d->listView->itemDelegate())->clearChangedEntries();
     emit changed(false);
 }
 
@@ -372,17 +373,24 @@ void KPluginSelector::save()
         pluginEntry->cfgGroup.sync();
     }
 
+    static_cast<KPluginSelector::Private::PluginDelegate*>(d->listView->itemDelegate())->clearChangedEntries();
     emit changed(false);
 }
 
 void KPluginSelector::defaults()
 {
     bool isChanged = false;
+    auto delegate = static_cast<KPluginSelector::Private::PluginDelegate*>(d->listView->itemDelegate());
+    delegate->clearChangedEntries();
     for (int i = 0; i < d->pluginModel->rowCount(); i++) {
         const QModelIndex index = d->pluginModel->index(i, 0);
         PluginEntry *pluginEntry = static_cast<PluginEntry *>(index.internalPointer());
-        isChanged |= pluginEntry->pluginInfo.isPluginEnabled() != pluginEntry->pluginInfo.isPluginEnabledByDefault();
+        bool entryChanged = pluginEntry->pluginInfo.isPluginEnabled() != pluginEntry->pluginInfo.isPluginEnabledByDefault();
+        isChanged |= entryChanged;
         d->pluginModel->setData(index, pluginEntry->pluginInfo.isPluginEnabledByDefault(), Qt::CheckStateRole);
+        if (entryChanged) {
+            delegate->addChangedEntry(pluginEntry);
+        }
     }
 
     emit changed(isChanged);
@@ -403,6 +411,7 @@ bool KPluginSelector::isDefault() const
 
 void KPluginSelector::updatePluginsState()
 {
+    static_cast<KPluginSelector::Private::PluginDelegate*>(d->listView->itemDelegate())->clearChangedEntries();
     for (int i = 0; i < d->pluginModel->rowCount(); i++) {
         const QModelIndex index = d->pluginModel->index(i, 0);
         PluginEntry *pluginEntry = static_cast<PluginEntry *>(index.internalPointer());
@@ -452,6 +461,15 @@ KPluginSelector::Private::PluginModel::~PluginModel()
 {
 }
 
+static bool hasServiceNoDisplaySet(const KPluginInfo &pluginInfo)
+{
+QT_WARNING_PUSH
+QT_WARNING_DISABLE_CLANG("-Wdeprecated-declarations")
+QT_WARNING_DISABLE_GCC("-Wdeprecated-declarations")
+    return pluginInfo.service() && pluginInfo.service()->noDisplay();
+QT_WARNING_POP
+}
+
 void KPluginSelector::Private::PluginModel::addPlugins(const QList<KPluginInfo> &pluginList, const QString &categoryName, const QString &categoryKey, const KConfigGroup &cfgGroup, PluginLoadMethod pluginLoadMethod, bool manuallyAdded)
 {
     QList<PluginEntry> listToAdd;
@@ -476,7 +494,7 @@ void KPluginSelector::Private::PluginModel::addPlugins(const QList<KPluginInfo>
 
         if (!pluginEntryList.contains(pluginEntry) && !listToAdd.contains(pluginEntry) &&
                 (categoryKey.isEmpty() || !pluginInfo.category().compare(categoryKey, Qt::CaseInsensitive)) &&
-                (!pluginInfo.service() || !pluginInfo.service()->noDisplay())) {
+                (!hasServiceNoDisplaySet(pluginInfo))) {
             listToAdd << pluginEntry;
 
             if (!pluginSelector_d->showIcons && !pluginInfo.icon().isEmpty()) {
@@ -794,8 +812,12 @@ void KPluginSelector::Private::PluginDelegate::emitChanged(bool state)
 {
     const QModelIndex index = focusedIndex();
     PluginEntry *pluginEntry = index.model()->data(index, PluginEntryRole).value<PluginEntry *>();
-
-    emit changed(pluginEntry->pluginInfo.isPluginEnabled() != state);
+    if (pluginEntry->pluginInfo.isPluginEnabled() != state) {
+        changedEntries << pluginEntry;
+    } else {
+        changedEntries.remove(pluginEntry);
+    }
+    emit changed(!changedEntries.isEmpty());
 }
 
 void KPluginSelector::Private::PluginDelegate::slotAboutClicked()
@@ -874,8 +896,6 @@ void KPluginSelector::Private::PluginDelegate::configure(const QModelIndex& inde
     if (!moduleProxyList.isEmpty()) {
         QVBoxLayout *layout = new QVBoxLayout;
         layout->addWidget(mainWidget);
-        const int marginHint = configDialog.style()->pixelMetric(QStyle::PM_DefaultChildMargin);
-        layout->insertSpacing(-1, marginHint);
 
         QDialogButtonBox *buttonBox = new QDialogButtonBox(&configDialog);
         buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel | QDialogButtonBox::RestoreDefaults);
@@ -891,7 +911,7 @@ void KPluginSelector::Private::PluginDelegate::configure(const QModelIndex& inde
 
         if (configDialog.exec() == QDialog::Accepted) {
             for (KCModuleProxy *moduleProxy : qAsConst(moduleProxyList)) {
-                const QStringList parentComponents = moduleProxy->moduleInfo().service()->property(QStringLiteral("X-KDE-ParentComponents")).toStringList();
+                const QStringList parentComponents = moduleProxy->moduleInfo().property(QStringLiteral("X-KDE-ParentComponents")).toStringList();
                 moduleProxy->save();
                 for (const QString &parentComponent : parentComponents) {
                     emit configCommitted(parentComponent.toLatin1());
